      PROGRAM SETHL41
C=======================================================================
C      DATA GENERATING PROGRAM FOR HELM4Q.FOR
C   PROJECT: DRIVEN CAVITY FLOW   PROJECT NAME: HEL4DATA.QQQ
C    DOMAIN: SQUARE ( 1 X 1 )
C    ELEMENT: FOUR-NODED ISOPARAMETRIC ELEMENT
C    DOMAIN DISCRETIZATION: UNEVEN ELEMENTS WITH VERTICAL SCAN
C     EIJI FUKUMORI DECEMBER 29, 1993 
C=======================================================================
      PARAMETER ( ND=4, MXE=20000, MXN=30000, MXB=3000 )
      PARAMETER (   TLX = 2.3,   TLY = 1.2 )
C=======================================================================
      DIMENSION NODEX(MXE,ND),XCOORD(MXN), YCOORD(MXN),IBNDF(MXB),
     * BV(MXB), ITYPE(MXB)
      CHARACTER PROJECT*12, EXFILE*3
      LOGICAL YES
C=======================================================================
      DATA PROJECT / 'HEL4DATA.QQQ' /
C=======================================================================
C      F(X) = X * X * ( 3.- 2.* X )
C      G(X) = X * ( (0.1-1.)* X + (2.- 0.1) )
      F(X) = X
      G(X) = X
C=======================================================================
      WRITE (*,*)' INPUT DATA FILE = ', PROJECT
C=======================================================================
C         NEY: NEMBER OF VERTICAL ELEMENTS (NUMBER OF NODES: NEY+1)
C         NEX: NEMBER OF HORIZONTAL ELEMENTS (NUMBER OF NODES: NEX+1)
C=======================================================================
      WRITE (*,210)
  210 FORMAT( ' TYPE IN NUMBER OF ELEMENTS IN X-COORD (NEX) = ' $ )
      READ (*,*) NEX
      IF ( NEX .LT. 1 ) STOP 'NOT ACCEPTABLE NEX'
      WRITE (*,220)
  220 FORMAT( ' TYPE IN NUMBER OF ELEMENTS IN Y-COORD (NEY) = '  $ )
      READ (*,*) NEY
      IF ( NEY .LT. 1 ) STOP 'NOT ACCEPTABLE NEY'
      WRITE(*,*)' NEX =',NEX, '     NEY =', NEY
C=======================================================================
      ALPHA = 8.7211D0
      DX = TLX / NEX
      DY = TLY / NEY
      NDX=NEX+1
      NDY=NEY+1
C=======================================================================
C                    ELEMENT CREATION
      NE = 0
      DO J = 1 , NEX
      DO I = 1 , NEY
      NE = NE + 1
      IF ( NE .GT. MXE ) STOP 'NE > MXE'
      NODEX(NE,1) = NDY*(J-1) + I
      NODEX(NE,2) = NODEX(NE,1) + NDY
      NODEX(NE,3) = NODEX(NE,2) + 1
      NODEX(NE,4) = NODEX(NE,1) + 1
      END DO
      END DO
C=======================================================================
C                 NODAL COORDINATE CREATION
      NNODE = 0
      DO I = 1 , NDX
      DO J = 1 , NDY
      NNODE = NNODE + 1
      IF ( NNODE .GT. MXN ) STOP 'NNODE > MXN'
      XCOORD(NNODE) = TLX * F( DX*(I-1)/TLX )
      YCOORD(NNODE) = TLY * G( DY*(J-1)/TLY )
      END DO
      END DO
C=======================================================================
C                    BOUNDARY CONDITIONS
      NBF = 1
      IBNDF(NBF) = NNODE
      ITYPE(NBF) = 1
      BV(NBF) =1.
C--------- FACE OF -X
C      NBF = 0
C      NDYPRT1 = NDY/3
CCCCCCC      DO J = 2 , NDY-1
C      DO J = 1, NDYPRT1
C      NBF = NBF + 1
C      IBNDF(NBF) = J
C      ITYPE(NBF) = 1
C      BV(NBF) = WAVEH
C      END DO
C      DO J = NDYPRT1+1,NDY-1
C      NBF = NBF + 1
C      IBNDF(NBF) = J
C      ITYPE(NBF) = 1
C      BV(NBF) = 0.
C      END DO
C--------- FACE OF -Y
C      DO I = 1 , NDX
C      NBF = NBF + 1
C      IBNDF(NBF) = 1 + NDY*(I-1)
C      ITYPE(NBF) = 1
C      BV(NBF) = 0.
C      END DO
C--------- FACE OF +Y
C      DO I = 1 , NDX
C      NBF = NBF + 1
C      IBNDF(NBF) = I*NDY
C      ITYPE(NBF) = 1
C      BV(NBF) = 0.
C      END DO
C--------- FACE OF +X
C      DO I = 2 , NEY
C      NBF = NBF + 1
C      IBNDF(NBF) = NEX*NDY + I
C      ITYPE(NBF) = 1
C      BV(NBF) = 0.
C      END DO
C=======================================================================
C                      DATA FILE INQUIRY
      EXFILE = 'NEW'
      INQUIRE ( FILE = PROJECT, EXIST = YES )
      IF ( YES ) EXFILE='OLD'
C=======================================================================
C                     MAKING DATA FILES
      OPEN ( 1, FILE=PROJECT, STATUS = EXFILE )
C------ CONTROL PARAMETERS
      WRITE(1,*) ALPHA
      WRITE(1,'(I5)' ) NE
C------ ELEMENTS
      DO I = 1 , NE
      WRITE (1,'(5I5)') I, (NODEX(I,J), J = 1 , ND )
      END DO
C------ NODAL COORDINATES
      WRITE(1,'(I5)'  ) NNODE
      WRITE(1,120) ( I,XCOORD(I), YCOORD(I) , I = 1, NNODE )
  120 FORMAT ( I5, 2F10.6 )
C------ BOUNDARY CONDITIONS
      WRITE(1,'(I5)'  ) NBF
      WRITE (1,125) ( IBNDF(I), ITYPE(I),BV(I) , I = 1 , NBF )
  125 FORMAT ( 2I5, F10.5 )
      CLOSE (1)
      STOP
      END
